"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getUploadDirFileList = exports.Storage = void 0;
const cos_nodejs_sdk_v5_1 = __importDefault(require("cos-nodejs-sdk-v5"));
const s3_1 = __importDefault(require("aws-sdk/clients/s3"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const path_1 = __importDefault(require("path"));
const mime_types_1 = require("mime-types");
class Storage {
    constructor({ endpoint, region, credential, cos = true, }) {
        this._cos = cos;
        this._region = region || '';
        if (this._cos) {
            this._instance = (credential === null || credential === void 0 ? void 0 : credential.token)
                ? new cos_nodejs_sdk_v5_1.default({
                    getAuthorization(options, callback) {
                        callback({
                            TmpSecretId: (credential === null || credential === void 0 ? void 0 : credential.secretId) || '',
                            TmpSecretKey: (credential === null || credential === void 0 ? void 0 : credential.secretKey) || '',
                            XCosSecurityToken: (credential === null || credential === void 0 ? void 0 : credential.token) || '',
                            ExpiredTime: Math.floor(Date.now() / 1000) + 600,
                            StartTime: Math.floor(Date.now() / 1000),
                        });
                    },
                })
                : new cos_nodejs_sdk_v5_1.default({
                    SecretId: credential === null || credential === void 0 ? void 0 : credential.secretId,
                    SecretKey: credential === null || credential === void 0 ? void 0 : credential.secretKey,
                });
        }
        else {
            this._instance = new s3_1.default({
                endpoint: endpoint || 'http://oss-csp1.rgw.infra.tce.io',
                region: region || 'us-east-1',
                s3ForcePathStyle: true,
                sslEnabled: false,
                credentials: {
                    accessKeyId: credential.secretId,
                    secretAccessKey: credential.secretKey,
                    sessionToken: credential.token || '',
                },
            });
        }
    }
    async getObjectSignedUrl({ Bucket, Key, Expires }) {
        const baseParam = { Bucket, Key, Expires };
        if (!this._cos) {
            const url = await this._instance.getSignedUrl('getObject', { ...baseParam });
            return { Url: url };
        }
        else {
            return new Promise((resolve, reject) => {
                this._instance.getObjectUrl({
                    ...baseParam,
                    Sign: true,
                    Region: this._region || 'ap-shanghai',
                }, (err, data) => {
                    if (err) {
                        reject(err);
                    }
                    else {
                        resolve(data);
                    }
                });
            });
        }
    }
    async putObject({ Bucket, Key, Body, ACL, }) {
        return new Promise((resolve, reject) => {
            const baseParam = {
                Bucket,
                Body,
                Key,
            };
            if (this._cos) {
                this._instance.putObject({
                    Region: this._region || 'ap-shanghai',
                    ...baseParam,
                }, (err) => {
                    if (err) {
                        reject(err);
                    }
                    else {
                        resolve(null);
                    }
                });
            }
            else {
                const contentType = (0, mime_types_1.lookup)(baseParam.Key);
                this._instance.putObject({
                    ACL,
                    ContentType: contentType || 'application/octet-stream',
                    ...baseParam,
                }, (err) => {
                    if (err) {
                        reject(err);
                    }
                    else {
                        resolve(null);
                    }
                });
            }
        });
    }
    async putBucketAcl({ ACL, Bucket }) {
        const baseParam = {
            Bucket,
        };
        return new Promise((resolve, reject) => {
            if (this._cos) {
                throw new Error('');
            }
            else {
                this._instance.putBucketAcl({
                    ACL,
                    ...baseParam,
                }, (err) => {
                    if (err) {
                        reject(err);
                    }
                    else {
                        resolve(null);
                    }
                });
            }
        });
    }
    async putObjectAcl({ ACL, Bucket, Key }) {
        const baseParam = {
            Bucket,
            Key,
        };
        return new Promise((resolve, reject) => {
            if (this._cos) {
                throw new Error('');
            }
            else {
                this._instance.putObjectAcl({
                    ACL,
                    ...baseParam,
                }, (err) => {
                    if (err) {
                        reject(err);
                    }
                    else {
                        resolve(null);
                    }
                });
            }
        });
    }
}
exports.Storage = Storage;
function getUploadDirFileList(src, cloudPath) {
    const stat = fs_extra_1.default.statSync(src);
    let uploadFilePathList = [];
    if (stat.isFile()) {
        uploadFilePathList.push({
            src,
            cloudPath,
        });
    }
    else if (stat.isDirectory()) {
        const files = fs_extra_1.default.readdirSync(src);
        for (const file of files) {
            uploadFilePathList = uploadFilePathList.concat(getUploadDirFileList(path_1.default.posix.join(src, file), path_1.default.posix.join(cloudPath, file)));
        }
    }
    return uploadFilePathList;
}
exports.getUploadDirFileList = getUploadDirFileList;
